/*
  Elex Prolog code generator module.

  (c) Gertjan van Noord 1997.
  based on work by Matt Phillips.
  */

#include <iostream.h>
#include <fstream.h>
#include <string.h>
#include <errno.h>
#include <stdlib.h>

#include <foo/FooParser.h>

#include "PrologCodeGenerator.h"

void error (const string message, int abortcode = 0)
{
  cerr << "generator: " << message << endl;
  if (abortcode > 0) exit (abortcode);
}

// check stream <str> which should be a stream opened for file with
// <name> is open.
int checkStream (ios &str, const string &name)
{
  if (!str)
  {
    error ("could not open '" + name + "': " + strerror (errno), 1);
  }
}

int main (int argc, char **argv)
{
  if (argc != 6)
  {
    cerr << "\nusage: generator <ecf> <prolog> <compact_code:{0,1}>\
<use_module:{0,1}> <lookahead:{int}>\n\n";
    return 1;
  }

  ifstream codefrags (argv [1]); checkStream (codefrags, argv [1]);
  ofstream body (argv [2]); checkStream (body, argv [2]);

  BufferedInputStream input (cin);
  FooParser parser (input);

  int option_compact_code = atoi(argv [3]);
  int option_use_module = atoi(argv [4]);
  int option_lookahead= atoi(argv [5]);

  /*
    cerr << "\ncompact_code: " << option_compact_code <<
            "\nuse_module: " << option_use_module <<
            "\nlookahead: " << option_lookahead << endl;
  */

  if (parser.nErrors () > 0)
  {
    error ("code generator: errors found in foo:");
    cerr << parser.getErrors () << endl;
    return 1;
  } else
  {
    if (!parser.getObject ().isAtomic ())
    {
      PrologCodeGenerator codegen (parser.getObject (),
				   codefrags, 
				   body, 
				   option_compact_code,
				   option_use_module,
				   option_lookahead
				   );
      return codegen.nErrors () != 0;
    } else
    {
      error ("illegal scanner definition", 1);
    }
  }
  return 0;
}
