/*
  $Id: BufferedInputStream.h,v 1.7 1997/02/02 02:11:00 matt Exp $

  Buffered input streams class.
  
  Adds basic buffering to InputStream class.
  
  (c) Feb 96 Matt Phillips.
  */

#ifndef _BINPSTR_H
#define _BINPSTR_H

#include <std/string.h>

#include <util/checks.h>

#include "InputStream.h"

class BufferedInputStream : public InputStream
{
public:

  BufferedInputStream (istream &s, string &fname)
    : InputStream (s, fname) {}

  BufferedInputStream (istream &s) 
    : InputStream (s) {}

  // get next char
  virtual void get ();

  // skip to next char (no buffering of current char)
  void skip () {InputStream::get ();}

  const string &getBuffer () const {return buffer;}

  // get char at buffer postion pos.
  char getBuffer (int pos)
  {
    CHECK (pos >= 0 && pos < buffer.length (), "access outside buffer");
    return buffer [pos];
  }

  // set the buffer to be 'chr'.
  void setBuffer (char chr)
  {
    buffer = chr;
  }

  // push chr into end of buffer
  void pushBuffer (char chr) {buffer += chr;}

  void clearBuffer () {buffer.assign (0, char (0));}

  int getBufferSize () const {return buffer.length ();}

protected:
   
  string buffer;
};

#endif
