/*
  $Id: ErrorMessage.cpp,v 1.4 1997/04/02 12:26:44 matt Exp $
  
  Error/warning messages.
  
  (c) 1996 Matt Phillips.
  */

#include "ErrorMessage.h"

// Add a error message.
void ErrorMessageList::error (const string &msg, const string &file,
			      int line, int column)
{
  addTail (*new ErrorMessage (msg, file, line, column,
			      ErrorMessage::Error));
  numErrors++;
}

// Add a warning message.
void ErrorMessageList::warning (const string &msg, const string &file,
				int line, int column)
{
  addTail (*new ErrorMessage (msg, file, line, column,
			      ErrorMessage::Warning));
  numWarnings++;
}

// Clear all messages.
void ErrorMessageList::clear ()
{
  TypeIOLinkedListWithTail(ErrorMessage)::clear ();
  numWarnings = numErrors = 0;
}

// Merge this message list with <list> (this list is cleared).
void ErrorMessageList::mergeWith (ErrorMessageList &list)
{
  moveTo (list);
  list.numErrors += numErrors;
  list.numWarnings += numWarnings;
  numWarnings = numErrors = 0;
}

ostream &operator << (ostream &s, const ErrorMessage &e)
{
  if (e.filename.length () > 0)
    s << e.filename << ':';

  s << e.line << ':' << e.column << ": " <<
    (e.messageType == ErrorMessage::Error ? "error" : "warning") <<
    ": " << e.message;

  return s;
}

template class LinkedListImp<ErrorMessage, ILinkedItem<ErrorMessage, 1> >;
template ostream &operator<<(ostream &, Container<ErrorMessage> const &);
template class LinkedListWithTailImp<ErrorMessage, ILinkedItem<ErrorMessage, 1> >;
template class Container<ErrorMessage>;
template class IContainerData<ErrorMessage, 1>;
template class ILinkedItem<ErrorMessage, 1>;
template class LinkedListImpIter<ErrorMessage, ILinkedItem<ErrorMessage, 1> >;
template class List<ErrorMessage>;
template class ListIter<ErrorMessage>;
template class ContainerIter<ErrorMessage>;
template class LinkedListWithTailImpIter<ErrorMessage, ILinkedItem<ErrorMessage, 1> >;
