/*
  $Id: AssociationItem.h,v 1.1 1996/04/20 07:53:36 matt Exp $

  Association data items,

  (c) May 95 Matt Phillips.
  */

#ifndef _ASSOCITM_H
#define _ASSOCITM_H

template <class K>
struct DAssocKey
{
  DAssocKey (const K &k) : key (k) {}

  int operator == (const DAssocKey<K> &k) const
  {return key == k.key;}

  int operator < (const DAssocKey<K> &k) const
  {return key < k.key;}

  const K &refKey () const {return key;}
   
  K key;
};

template <class K>
class IAssocKey
{
public:

  IAssocKey (const K &k) : key (&k) {}

  int operator == (const IAssocKey<K> &k) const
  {return *key == *k.key;}

  int operator < (const IAssocKey<K> &k) const
  {return *key < *k.key;}

  const K &refKey () const {return *key;}

  const K *key;
};

template <class K, class T>
struct DAssocItem : public DAssocKey<K>
{
  DAssocItem (const K &k, const T &d) :  DAssocKey<K> (k), data (d) {}

  T &ref () {return data;}
  void set (T &d) {data = d;}
   
  T data;
};

template <class K, class T, int ownsT, int ownsK>
class IAssocItem : public IAssocKey<K>
{
public:

  IAssocItem (const K &k, T &d) : IAssocKey<K> (k), data (&d) {}

  ~IAssocItem () {if (ownsT) delete data; if (ownsK) delete key;}

  T &ref () {return *data;}
  void set (T &d) {data = &d;}

  T *data;
};

template <class K, class T, int ownsK>
struct IKAssocItem : public IAssocKey<K>
{
  IKAssocItem (const K &k, const T &d) : IAssocKey<K> (k), data (d) {}
  ~IKAssocItem () {if (ownsK) delete key;}

  T &ref () {return data;}

  void set (T &d) {data = d;}

  T data;
};

template <class K, class T, int ownsT>
struct ITAssocItem : public DAssocKey<K>
{
  ITAssocItem (const K &k, T &d) : DAssocKey<K> (k), data (&d) {}
  ~ITAssocItem () {if (ownsT) delete data;}

  T &ref () {return *data;}

  void set (T &d) {data = &d;}

  T *data;
};

#endif
